package data;

import javax.media.opengl.GL;

/**
 * Camera in OpenGL-context
 * @author Felix Flei (MatrNr: 0527038)
 *
 */
public class Camera {
	private boolean moveUp;
	private boolean moveDown;
	private boolean moveLeft;
	private boolean moveRight;
	private boolean moveForward;
	private boolean moveBackward;
	private double pan;
	private double tilt;
	private double zoom;
	private double[] pos;
	private double speed;
	private GL gl;
	/**
	 * initializes Camera parameters
	 * @param pos absolute position
	 * @param pan pan of the camera
	 * @param tilt tilt of the camera
	 * @param speed movement speed of the camera
	 * @param gl GL-context
	 */
	public Camera(double[] pos, double pan, double tilt, double zoom, double speed, GL gl){
		this.pos = pos;
		this.pos[0] *=-1;
		this.pos[1] *=-1;
		this.pos[2] *=-1;
		this.speed = speed;
		this.gl = gl;
		this.pan = pan;
		this.tilt = tilt;
		this.zoom = zoom;
		this.moveUp = false;
		this.moveDown = false;
		this.moveLeft = false;
		this.moveRight = false;
		this.moveForward = false;
		this.moveBackward = false;
	}
	/**
	 * Moves the camera up when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveUp(boolean b){
		this.moveUp = b;
	}
	/**
	 * Moves the camera down when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveDown(boolean b){
		this.moveDown = b;
	}
	/**
	 * Moves the camera left when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveLeft(boolean b){
		this.moveLeft = b;
	}
	/**
	 * Moves the camera right when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveRight(boolean b){
		this.moveRight = b;
	}
	/**
	 * Moves the camera forward when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveForward(boolean b){
		this.moveForward = b;
	}
	/**
	 * Moves the camera backward when {@link #update(long deltaTime)} is called
	 * @param b
	 */
	public void moveBackward(boolean b){
		this.moveBackward = b;
	}
	/**
	 * Pans the camera when {@link #update(long deltaTime)} is called
	 * @param pan pan to add
	 */
	public void addPan(double pan){
		this.pan -= pan;
	}
	/**
	 * simply moves the camera instantly plus the given vector
	 * @param x
	 * @param y
	 */
	public void move(float x, float y, float z){
		this.pos[0] += x;
		this.pos[1] += y;
		this.pos[2] += z;
	}
	/**
	 * returns the zoom factor of the camera
	 * @return zoom
	 */
	public double getZoom() {
		return zoom;
	}
	/**
	 * returns the camera position in world coordinates
	 * @return position
	 */
	public double[] getPos() {
		return pos;
	}
	/**
	 * Tilts the camera when {@link #update(long deltaTime)} is called
	 * @param tilt tilt to add
	 */
	public void addTilt(double tilt){
		this.tilt -= tilt;
		
	}
	/**
	 * Tilts the camera when {@link #update(long deltaTime)} is called
	 * @param zoom
	 */
	public void addZoom(double zoom){
		this.zoom += zoom;
		if(this.zoom<0.001)this.zoom=0.001;
		if(this.zoom>1)this.zoom=1.0;
		
	}
	/**
	 * updates the camera position and orientation
	 * @param deltaTime passed time since {@link #update(long deltaTime)}
	 * was called the last time
	 */
	public void update(long deltaTime){
		double dist = speed*(double)deltaTime;
		if(moveBackward){
			pos[2] -= dist*Math.cos(this.pan*2.0*Math.PI/360.0);
			pos[0] -= dist*Math.sin(this.pan*2.0*Math.PI/360.0);
		}
		if(moveForward){
			pos[2] += dist*Math.cos(this.pan*2.0*Math.PI/360.0);
			pos[0] += dist*Math.sin(this.pan*2.0*Math.PI/360.0);
		}
		if(moveLeft){
			pos[2] -= dist*Math.sin(this.pan*2.0*Math.PI/360.0);
			pos[0] += dist*Math.cos(this.pan*2.0*Math.PI/360.0);
		}
		if(moveRight){
			pos[2] += dist*Math.sin(this.pan*2.0*Math.PI/360.0);
			pos[0] -= dist*Math.cos(this.pan*2.0*Math.PI/360.0);
		}
		if(moveUp)pos[1] -= dist;
		if(moveDown)pos[1] += dist;
		
	}
	/**
	 * transforms the OpenGL objects into camera space.
	 * NOTE: call this before drawing
	 */
	public void transformIntoCameraSpace(){
		this.gl.glRotated(-this.tilt, 1.0, 0.0, 0.0);
		this.gl.glRotated(-this.pan, 0.0, 1.0, 0.0);
		this.gl.glScaled(zoom, zoom, 1.0);
		this.gl.glTranslated(pos[0], pos[1], pos[2]);
	}
}
